@@ -34,25 +34,23 @@
 
 #define TAPP_ERR_FLDD	1		/* ΥեȥåȤѤ */
 
-#define TFONT_TYPE_U	0		/* UNKNOWN */
-#define TFONT_TYPE_S	1		/* SINGLE */
-#define TFONT_TYPE_D	2		/* DOUBLE */
-
-#define TFONT_FT_SINGLE		0x00000000	/* 1 byte ʸå */
-#define TFONT_FT_DOUBLE		0x01000000	/* 2 byte ʸå */
+#define TFONT_FT_SINGLE		0x00000000	/* 1 byte character set */
+#define TFONT_FT_DOUBLE		0x01000000	/* 2 byte character set */
 #define TFONT_FT_94CHAR		0x00000000	/* 94 or 94^n */
 #define TFONT_FT_96CHAR		0x02000000	/* 96 or 96^n */
+#define TFONT_FT_OTHER		0x10000000	/* other coding system */
 
 #define TFONT_OWNER		0x00	/* ι¤Τglyph */
 #define TFONT_ALIAS		0x01	/* ̤ι¤Τglyph򻲾 */
 
 typedef enum {
 	FH_LEFT,
-	FH_RIGHT
+	FH_RIGHT,
+	FH_UNI,
 } FONTSET_HALF;
 
 typedef struct Raw_TFont {
-	const u_char* (*conv)(struct Raw_TFont* p, u_int c);
+	const u_char* (*conv)(struct Raw_TFont* p, u_int c, u_int* width);
 	/* --- */
 	const char* name;
 	u_int width;		/* ʸοʿɥåȿ */
@@ -63,6 +61,7 @@ typedef struct Raw_TFont {
 	u_char aliasF;
 	/* */
 	u_char** glyph;		/* ӥåȥޥåγglyph Ƭ */
+	u_int* glyph_width;
 	u_char* dglyph;		/* бglyph¸ߤʤglyph */
 	u_char* bitmap;		/* ӥåȥޥå */
 	u_int colf; 
@@ -75,18 +74,18 @@ typedef struct Raw_TFont {
 } TFont;
 
 
-const u_char* tfont_default_glyph(TFont* p, u_int c);
-const u_char* tfont_standard_glyph(TFont* p, u_int c);
+const u_char* tfont_default_glyph(TFont* p, u_int c, u_int *width);
+const u_char* tfont_standard_glyph(TFont* p, u_int c, u_int *width);
 void tfont_final(TFont* p);
 void tfont_ary_final(void);
 void tfont_init(TFont* p);
 
 void tfont_setup_fontlist(TCapValue* values);
 int tfont_is_valid(TFont* p);
+int tfont_is_loaded(TFont *p);
 
 int tfont_ary_search_idx(const char* na);
 void tfont_ary_show_list(FILE* fp);
-void tfont_show_sequence(FILE *fp, const char *en);
 
 extern TFont gFont[];
 extern u_int gFontsWidth;
