@@ -45,6 +45,7 @@
 #include <getopt.h>
 #include <locale.h>
 #include <langinfo.h>
+#include <iconv.h>
 
 #include "pcf.h"
 #include "font.h"
@@ -342,24 +343,24 @@ tapp_setup_encoding(char *en)
 	}
 	if (strchr(en, ',') == NULL) {
 		char *encname = en;
-		en = tcaps_find_entry(&(gApp.gCaps), "encoding.", en);
-		if (!en)
-			print_error("%s not found, fallback to default\n", encname);
+		en = tcaps_find_entry(&(gApp.gCaps), "encoding.", encname);
+		if (!en) {
+			/* not found */
+			iconv_t cd = iconv_open("UCS-2BE", encname);
+			if (cd != (iconv_t)(-1)) {
+				iconv_close(cd);
+				en = (char *)malloc(strlen("other,iconv,UTF-8")+1+strlen(encname));
+				if (en == NULL)
+					die("not enough memory: encode");
+				sprintf(en, "other,%s,iconv,UTF-8", encname);
+			} else {
+				print_error("%s not found, fallback to default\n", encname);
+			}
+		}
 	}
 	if (!en) {
 		en = "G0,G1,ansix3.4-1968,ansix3.4-1968,iso8859.1-1987,ansix3.4-1968";
 	}
-#ifdef JFB_OTHER_CODING_SYSTEM
-	{
-		/* XXX : initialize other-coding-system */
-		TCapability* fother;
-		fother = tcaps_find(&(gApp.gCaps), "other-coding-system");
-		if (fother && fother->values) {
-			tvterm_setup_otherCodingSystem(&(gApp.gCaps), 
-						       fother->values);
-		}
-	}
-#endif
 	return en;
 }
 
