@@ -182,13 +182,32 @@ void tpcfformat_load(TPcfFormat* p, FILE* fp)
 {
 	u_int t = read_u32l(fp);
 #if PCF_DEBUG
-	printf("[[FORMAT: %x]]\n", t);
+	printf("[[FORMAT: %x = ", t);
 #endif
 	p->id = (t >> 8) & 0xfff;
 	p->scan = 1 << ((t >> 4) & 0x3);
 	p->obit = (t >> 3) & 1;
 	p->obyte = (t >> 2) & 1;
 	p->glyphaline = t & 0x3;
+#if PCF_DEBUG
+	switch (p->id) {
+	case TPCF_DEFAULT_FORMAT: printf("default "); break;
+	case TPCF_INKBOUNDS: printf("inkbounds "); break;
+	case TPCF_ACCEL_W_INKBOUNDS: printf("accel/ink|compr/metric "); break;
+	default: printf("???"); break;
+	}
+	printf("scan:%d bytes ", p->scan);
+	if (p->obit)
+		printf("MSB ");
+	else
+		printf("LSB ");
+	if (p->obyte)
+		printf("BE ");
+	else
+		printf("LE ");
+	printf("g:%d (%d bytes)", p->glyphaline, 1<<p->glyphaline);
+	printf("]]\n");
+#endif
 }
 
 /*---------------------------------------------------------------------------*/
@@ -208,8 +227,21 @@ void tpcfindex_load(TPcfIndex* p, FILE* fp)
 
 void tpcfindex_debug(TPcfIndex* p)
 {
-	printf("type=%u, size=%u, offset=%u",
-	 	p->type, p->size, p->offset);
+	printf("type=%u:", p->type);
+	switch (p->type) {
+	case TPCF_PROPERTIES: printf("prop "); break;
+	case TPCF_ACCELERATORS: printf("accel "); break;
+	case TPCF_METRICS: printf("metric "); break;
+	case TPCF_BITMAPS: printf("bitmaps "); break;
+	case TPCF_INK_METRICS: printf("inkmetric "); break;
+	case TPCF_BDF_ENCODINGS: printf("bdfencodings "); break;
+	case TPCF_SWIDTHS: printf("swidth "); break;
+	case TPCF_GLYPH_NAMES: printf("glyphnames "); break;
+	case TPCF_BDF_ACCELERATORS: printf("bdfaccel "); break;
+	default: printf("???"); break;
+	}
+	printf(", size=%u, offset=%u",
+	 	p->size, p->offset);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -286,15 +318,23 @@ size_t tpcfmetric_load(TPcfMetric* p, FILE* fp, int e)
 	p->leftsb = read_s16(fp, e);
 	p->rightsb = read_s16(fp, e);
 	p->width = read_s16(fp, e);
-#if 0
-	printf(">>>> %d\n", p->width);
-#endif
 	p->ascent = read_s16(fp, e);
 	p->descent = read_s16(fp, e);
 	p->attr = read_s16(fp, e);
 	return 2*6;
 }
 
+size_t tpcfcmetric_load(TPcfMetric* p, FILE* fp, int e)
+{
+	p->leftsb = read_u8(fp) - 0x80;
+	p->rightsb = read_u8(fp) - 0x80;
+	p->width = read_u8(fp) - 0x80;
+	p->ascent = read_u8(fp) - 0x80;
+	p->descent = read_u8(fp) - 0x80;
+	p->attr = 0;
+	return 1*5;
+}
+
 void tpcfmetric_debug(TPcfMetric* p)
 {
 	printf("[METRIC:%d<>%d(%d):%d^v%d:%d]",
@@ -302,22 +342,91 @@ void tpcfmetric_debug(TPcfMetric* p)
 		p->ascent, p->descent, p->attr);
 }
 
+
+void tpcfmetrics_init(TPcfMetrics* p) {
+	p->nMetrics = 0;
+	p->metric = NULL;
+}
+
+void tpcfmetrics_final(TPcfMetrics* p) {
+	p->nMetrics = 0;
+	free(p->metric);
+}
+
+size_t tpcfmetrics_load(TPcfMetrics* p, FILE* fp)
+{
+	TPcfFormat fmt;
+	int i;
+	tpcfformat_load(&fmt, fp);
+	size_t r = 0;
+
+	if (fmt.id == TPCF_DEFAULT_FORMAT) {
+		p->nMetrics = read_s32(fp, fmt.obit);
+#ifdef PCF_DEBUG
+		printf("nmetric(default): %d\n", p->nMetrics);
+#endif
+		if ((p->metric = (TPcfMetric*)malloc(sizeof(TPcfMetric) * p->nMetrics)) == NULL) {
+			die("(FONT): malloc error (metrics table)\n");
+		}
+		for (i = 0; i < p->nMetrics; i++) {
+			r += tpcfmetric_load(&(p->metric[i]), fp, fmt.obit);
+		}
+		return 8 + r;
+	} else if (fmt.id == TPCF_COMPRESSED_METRICS) {
+		p->nMetrics = read_u16(fp, fmt.obit);
+#ifdef PCF_DEBUG
+		printf("nmetric(compressed): %d\n", p->nMetrics);
+#endif
+		if ((p->metric = (TPcfMetric*)malloc(sizeof(TPcfMetric) * p->nMetrics)) == NULL) {
+			die("(FONT): malloc error (metrics table)\n");
+		}
+		for (i = 0; i < p->nMetrics; i++) {
+			r += tpcfcmetric_load(&(p->metric[i]), fp, fmt.obit);
+		}
+		return 6 + r;
+	}
+	return 4;
+}
+
+
 /*---------------------------------------------------------------------------*/
 size_t tpcfaccel_load(TPcfAccel* p, FILE* fp)
 {
-	u_char termf;
 	TPcfFormat fmt;
 	size_t r;
 
 	tpcfformat_load(&fmt, fp);
+
+	/* bool8: noOverlap */
+	/* bool8: constantMetrics */
 	skip_nbyte(fp, 2);
-	termf = read_u8(fp);
-	if (!termf) {
-		die("(FONT) : Not terminal font.\n");
+
+	/* bool8: terminalFont */
+	p->termf = read_u8(fp);
+#if 0 /* XXX: unifont is not terminal font */
+	if (!p->termf) {
+		die("(FONT) : Not terminal font %02x.\n", p->termf);
 	}
+#endif
+	/* bool8: contantWidth */
+	/* bool8: inkInside */
+	/* bool8: inkMetrics */
+	/* bool8: drawDirection */
+	/* bool8: dummy (padding) */
+	/* int32: fontAscent */
+	/* int32: fontDescent */
+	/* int32: maxOverlap */
 	skip_nbyte(fp, 5+4*3);
+
+	/* metric: minBounds */
 	r = tpcfmetric_load(&(p->metric), fp, fmt.obit);
+
+	/* metric: maxBounds */
 	
+	/* if format.id == PCF_ACCEL_W_INKBOUNDS */
+	/*   metric: ink_minBounds */
+	/*   metric: ink_maxBounds */
+	/* endif */
 	return 24+r;
 }
 
@@ -325,7 +434,7 @@ void tpcfaccel_debug(TPcfAccel* p)
 {
 	printf("[ACCEL:");
 	tpcfmetric_debug(&(p->metric));
-	printf("]");
+	printf("]\n");
 }
 
 /*---------------------------------------------------------------------------*/
@@ -388,7 +497,8 @@ size_t tpcfbitmap_load(TPcfBitmap* p, FILE* fp)
 	}
 	r = p->mapsize[fmt.glyphaline];
 #if PCF_DEBUG
-	printf("BITMAP SIZE : %d - %d\n", r, fmt.glyphaline);
+	printf("BITMAP SIZE : %d bytes - %d (%d bytes/line)\n", r, fmt.glyphaline, 
+	       p->aline);
 #endif
 	if (!(p->bitmap = (u_char*)malloc(r))) {
 		die("(FONT): malloc error(bitmap)\n");
@@ -434,11 +544,21 @@ size_t tpcfencode_load(TPcfEncode* p, FILE* fp)
 	return 14+2*r;
 }
 
+void tpcfencode_debug(TPcfEncode* p)
+{
+	printf("ENCODE[");
+	printf("col:%d-%d[%x-%x]", p->colf, p->coll, p->colf, p->coll);
+	printf("row:%d-%d[%x-%x]", p->rowf, p->rowl, p->rowf, p->rowl);
+	printf("defch: %d ", p->defa);
+	printf("n = %d]\n", (p->coll - p->colf + 1) * (p->rowl - p->rowf + 1));
+}
+
 /*---------------------------------------------------------------------------*/
 void tpcf_init(TPcf* p) {
 	p->nIdx = 0;
 	p->idxs = NULL;
 	tpcfprops_init(&(p->props));
+	tpcfmetrics_init(&(p->metrics));
 	tpcfbitmap_init(&(p->bitmap));
 	tpcfencode_init(&(p->encode));
 }
@@ -446,6 +566,7 @@ void tpcf_init(TPcf* p) {
 void tpcf_final(TPcf* p) {
 	free(p->idxs);
 	tpcfprops_final(&(p->props));
+	tpcfmetrics_final(&(p->metrics));
 	tpcfbitmap_final(&(p->bitmap));
 	tpcfencode_final(&(p->encode));
 }
@@ -503,6 +624,7 @@ void tpcf_load(TPcf* p, FILE* fp)
 	readp = seek_section(fp, readp, q->offset);
 #if PCF_DEBUG
 	printf("total gain = %d\n", readp);
+	printf("load PROPS\n");
 #endif
 	readp += tpcfprops_load(&(p->props), fp);
 #if PCF_DEBUG
@@ -521,6 +643,7 @@ void tpcf_load(TPcf* p, FILE* fp)
 		readp = seek_section(fp, readp, q->offset);
 #if PCF_DEBUG
 		printf("total gain = %d:%ld\n", readp, ftell(fp));
+		printf("load ACCEL\n");
 #endif
 		readp += tpcfaccel_load(&(p->accel), fp);
 #if PCF_DEBUG
@@ -528,6 +651,18 @@ void tpcf_load(TPcf* p, FILE* fp)
 		printf("total gain = %d\n", readp);
 #endif
 	}
+	/* read metric: for multiwidth font (unifont) ? */
+	q = tpcf_search_section(p, TPCF_METRICS);
+	if (!q) die("(FONT): Metric Section not exist.\n");
+	readp = seek_section(fp, readp, q->offset);
+#if PCF_DEBUG
+	printf("total gain = %d\n", readp);
+	printf("load METRIC\n");
+#endif
+	readp += tpcfmetrics_load(&(p->metrics), fp);
+#if PCF_DEBUG
+	printf("total gain = %d\n", readp);
+#endif
 
 	/* read bitmap */
 	q = tpcf_search_section(p, TPCF_BITMAPS);
@@ -535,6 +670,7 @@ void tpcf_load(TPcf* p, FILE* fp)
 	readp = seek_section(fp, readp, q->offset);
 #if PCF_DEBUG
 	printf("total gain = %d\n", readp);
+	printf("load BITMAP\n");
 #endif
 	readp += tpcfbitmap_load(&(p->bitmap), fp);
 #if PCF_DEBUG
@@ -547,9 +683,11 @@ void tpcf_load(TPcf* p, FILE* fp)
 	readp = seek_section(fp, readp, q->offset);
 #if PCF_DEBUG
 	printf("total gain = %d:%ld\n", readp, ftell(fp));
+	printf("load ENCODE\n");
 #endif
 	readp += tpcfencode_load(&(p->encode), fp);
 #if PCF_DEBUG
+	tpcfencode_debug(&(p->encode));
 	printf("total gain = %d:%ld\n", readp, ftell(fp));
 #endif
 
@@ -559,6 +697,7 @@ void tpcf_load(TPcf* p, FILE* fp)
 	readp = seek_section(fp, readp, q->offset);
 #if PCF_DEBUG
 	printf("total gain = %d\n", readp);
+	printf("load BDF_ACCEL\n");
 #endif
 	readp += tpcfaccel_load(&(p->accel), fp);
 #if PCF_DEBUG
@@ -600,15 +739,30 @@ void tpcf_as_tfont(TPcf* p, TFont* q)
 
 	gs = (q->coll - q->colf +1) * (q->rowl  - q->rowf +1);
 	if (!(q->glyph = (u_char**)malloc(sizeof(u_char*) * gs))) {
-		die("(FONT): malloc error\n");
+		die("(FONT): malloc error (glyph)\n");
+	}
+	q->glyph_width = NULL;
+	if (p->accel.termf == 0) {
+		/* not terminal font */
+		q->glyph_width = (u_int*)malloc(sizeof(u_int)*gs);
+		if (q->glyph_width == NULL) {
+			die("(FONT): malloc error (glyph_width)\n");
+		}
 	}
 	for (i = 0 ; i < gs ; i++) {
 		ii = p->encode.table[i];
 		if (ii == 0xffff) {
 			q->glyph[i] = q->dglyph;
+			if (q->glyph_width)
+				q->glyph_width[i] = q->width; /* XXX */
 		} else {
 			offset = p->bitmap.offsets[ii];
 			q->glyph[i] = q->bitmap + offset;
+			if (q->glyph_width) {
+				TPcfMetric *m;
+				m = &p->metrics.metric[ii];
+				q->glyph_width[i] = m->rightsb - m->leftsb;
+			}
 		}
 	}
 }
@@ -627,3 +781,87 @@ const char* tpcf_get_string_prop(TPcf* p, const char* key)
 	return NULL;
 }
 
+#if PCF_DEBUG
+#include "main.h"
+TApplication gApp;
+
+/* cc -I. -o p -DPCF_DEBUG=1 pcf.c message.o */
+
+void
+dump_bits(int ch)
+{
+	int b;
+	/* printf("%02x:", ch); */
+	for (b = 7; b >= 0; --b) {
+		if ((ch & (1<<b)) != 0) {
+			printf("*");
+		} else {
+			printf("-");
+		}
+	}
+	printf(" ");
+}
+
+int
+main(int argc, char *argv[])
+{
+    FILE *fp;
+    char buf[512];
+    TPcf tpcf;
+    if (argc < 2) {
+	    fprintf(stderr, "usage: %s <fontfile> [idx]\n", argv[0]);
+	    exit(1);
+    }
+    sprintf(buf, "gunzip < %s", argv[1]);
+    fp = popen(buf, "r");
+    
+    tpcf_load(&tpcf, fp);
+    if (argc > 2) {
+	    int idx = strtol(argv[2], NULL, 0);
+	    int eidx;
+	    int offset;
+	    char *p;
+	    int i;
+	    int height, width;
+	    printf("idx:%d (0x%04x) => ", idx, idx);
+	    printf("colspan=%d ", 
+		    	(tpcf.encode.coll - tpcf.encode.colf + 1));
+	    eidx = ((idx >> 8) - tpcf.encode.rowf) * 
+		    	(tpcf.encode.coll - tpcf.encode.colf + 1)
+			+ (idx & 0x0ff) - tpcf.encode.colf;
+	    printf("=> %d (0x%04x)\n", eidx, eidx);
+	    offset = tpcf.encode.table[eidx];
+	    printf("encode.offset:%d (0x%04x)\n", offset, offset);
+	    if (offset == 0xffff)
+		    printf("no glyph\n");
+	    else {
+		    TPcfMetric *m;
+		    int mheight, mwidth;
+		    if (offset >= tpcf.metrics.nMetrics) {
+			    printf("out of error? metric: %d > %d\n",
+				   offset, tpcf.metrics.nMetrics);
+			    exit(1);
+		    }
+		    m = &tpcf.metrics.metric[offset];
+		    offset = tpcf.bitmap.offsets[offset];
+		    printf("bitmap.offset:%d\n", offset);
+		    p = &tpcf.bitmap.bitmap[offset];
+		    height = tpcf.accel.metric.ascent + tpcf.accel.metric.descent;
+		    width = tpcf.accel.metric.rightsb - tpcf.accel.metric.leftsb;
+		    mheight = m->ascent + m->descent;
+		    mwidth = m->rightsb - m->leftsb;
+		    printf("(h:%d * w:%d) h:%d * w:%d \n", 
+			   height, width, mheight, mwidth);
+		    for (i = 0; i < height; i++) {
+			    int j;
+			    for (j = 0; j < tpcf.bitmap.aline; j++) {
+				    dump_bits(*p & 0x0ff); p++;
+			    }
+			    printf("\n");
+		    }
+	    }
+    }
+    exit(0);
+    
+}
+#endif
