@@ -1,5 +1,6 @@
 /*
  * JFBTERM - 
+ * Copyright (C) 2003  Fumitoshi UKAI <ukai@debian.or.jp>
  * Copyright (C) 1999  Noritoshi MASUICHI (nmasu@ma3.justnet.ne.jp)
  *
  * KON2 - Kanji ON Console -
@@ -38,12 +39,12 @@
 #include	<sys/kd.h>
 #include	<errno.h>
 
+#include	"config.h"
 #include	"util.h"
 #include	"term.h"
 #include	"vterm.h"
 #include	"font.h"
 #include	"csv.h"
-#include	"config.h"
 
 #include	"sequence.h"
 #include	"message.h"
@@ -53,9 +54,13 @@
 static void tvterm_set_default_encoding(TVterm* p, const char* en);
 static void tvterm_esc_start(TVterm* p, u_char ch);
 static void tvterm_esc_bracket(TVterm*, u_char);
+#ifdef JFB_OTHER_CODING_SYSTEM
+static void tvterm_esc_rbracket(TVterm*, u_char);
+#endif
 static void tvterm_esc_traditional_multibyte_fix(TVterm* p, u_char ch);
+static int tvterm_find_font_index(int fsig);
 static void tvterm_esc_designate_font(TVterm* p, u_char ch);
-static void tvterm_esc_designate_other_coding_system(TVterm* p, u_char ch);
+static void tvterm_esc_designate_otherCS(TVterm* p, u_char ch);
 static void tvterm_invoke_gx(TVterm* p, TFontSpec* fs, u_int n);
 static void tvterm_re_invoke_gx(TVterm* p, TFontSpec* fs);
 void tvterm_set_window_size(TVterm* p);
@@ -67,12 +72,10 @@ static void tvterm_switch_to_UTF8(TVterm *p);
 static int tvterm_is_UTF8(TVterm *p);
 #endif
 #ifdef JFB_OTHER_CODING_SYSTEM
-static void tvterm_codingSystem_final(TVterm *p);
-static void tvterm_switch_to_other(TVterm *p, TCodingSystem *other);
-static int tvterm_is_otherCodingSystem(TVterm *p);
+static void tvterm_finish_otherCS(TVterm *p);
+static void tvterm_switch_to_otherCS(TVterm *p, TCodingSystem *other);
+static int tvterm_is_otherCS(TVterm *p);
 
-#define MAX_OTHER_CODING_SYSTEM		6
-static TCodingSystem gOtherCodingSystem[MAX_OTHER_CODING_SYSTEM];
 #endif
 
 void tvterm_init(TVterm* p, TTerm* pt, u_int hw, u_int hh, TCaps *caps, const char* en)
@@ -106,7 +109,7 @@ void tvterm_init(TVterm* p, TTerm* pt, u_int hw, u_int hh, TCaps *caps, const ch
 	p->ucs2ch = 0;
 #endif
 #ifdef JFB_OTHER_CODING_SYSTEM
-	p->codingSystem = NULL;
+	p->otherCS = NULL;
 #endif
 	p->caps = caps;
 	tvterm_set_default_encoding(p, en);
@@ -149,7 +152,7 @@ static void
 tvterm_switch_to_ISO2022(TVterm *p)
 {
 #ifdef JFB_OTHER_CODING_SYSTEM
-	tvterm_codingSystem_final(p);
+	tvterm_finish_otherCS(p);
 #endif
 #ifdef JFB_UTF8
 	p->utf8Idx = 0;
@@ -162,7 +165,7 @@ static int
 tvterm_is_ISO2022(TVterm *p)
 {
 #ifdef JFB_OTHER_CODING_SYSTEM
-	if (tvterm_is_otherCodingSystem(p))
+	if (tvterm_is_otherCS(p))
 		return 0;
 #endif
 #ifdef JFB_UTF8
@@ -205,7 +208,7 @@ static void
 tvterm_switch_to_UTF8(TVterm *p)
 {
 #ifdef JFB_OTHER_CODING_SYSTEM
-	tvterm_codingSystem_final(p);
+	tvterm_finish_otherCS(p);
 #endif
 	if (p->utf8DefaultIdx == 0) {
 		const char *en = tcaps_find_entry(p->caps,
@@ -231,71 +234,101 @@ tvterm_is_UTF8(TVterm *p)
 
 #ifdef JFB_OTHER_CODING_SYSTEM
 int
-tvterm_parse_other(const char *en, TCodingSystem* other)
+tvterm_parse_otherCS(const char *en, TCodingSystem* otherCS)
 {
 	TCsv farg;
 	const char *g;
 
+	memset(otherCS, 0, sizeof(*otherCS));
 	tcsv_init(&farg, en);
 
 	if (farg.cap != 4) {
 		tcsv_final(&farg);
 		return 0;
 	}
-	g = tcsv_get_token(&farg); /* other */
-	if (other->fromcode)
-		free(other->fromcode);
-	other->fromcode = NULL;
-	if (other->tocode)
-		free(other->tocode);
-	other->tocode = NULL;
+	g = tcsv_get_token(&farg); /* g == "other" */
 
 	g = tcsv_get_token(&farg);
-	other->fromcode = strdup(g);
+	otherCS->fromcode = strdup(g);
+
+	g = tcsv_get_token(&farg); /* g == "iconv" */
 
 	g = tcsv_get_token(&farg);
-	/* g == "iconv" */
+	otherCS->tocode = strdup(g);
 
-	g = tcsv_get_token(&farg);
-	other->tocode = strdup(g);
 	tcsv_final(&farg);
 
 	return 1;
 }
 
 static void
-tvterm_codingSystem_final(TVterm *p)
+tvterm_finish_otherCS(TVterm *p)
 {
 	/* XXX restore ISO-2022 state? */
-	if (p->codingSystem) {
-		if (p->codingSystem->cd != (iconv_t)(-1)) {
-			iconv_close(p->codingSystem->cd);
+	if (p->otherCS) {
+		TCodingSystem *o = p->otherCS;
+		int i;
+		/* restore designate/invoke state */
+		for (i = 0; i < 4; i++)
+			p->gIdx[i] = o->gSavedIdx[i];
+		tvterm_invoke_gx(p, &(p->gl), o->gSavedL);
+		tvterm_invoke_gx(p, &(p->gr), o->gSavedR);
+		p->tgl = p->gl;
+		p->tgr = p->gr;
+		p->knj1 = 0;
+		p->utf8Idx = o->utf8SavedIdx;
+		p->utf8remain = 0;
+		p->ucs2ch = 0;
+
+		if (o->cd != (iconv_t)(-1)) {
+			iconv_close(o->cd);
 		}
-		p->codingSystem->cd = (iconv_t)(-1);
-		if (strcmp(p->codingSystem->tocode, "UTF-8") != 0)
-			p->utf8Idx = 0;
+		o->cd = (iconv_t)(-1);
+		if (o->fromcode)
+			free(o->fromcode);
+		o->fromcode = NULL;
+		if (o->tocode)
+			free(o->tocode);
+		o->tocode = NULL;
 	}
-	p->codingSystem = NULL;
+	p->otherCS = NULL;
 }
 
 static void 
-tvterm_switch_to_other(TVterm *p, TCodingSystem *other)
+tvterm_switch_to_otherCS(TVterm *p, TCodingSystem *ocs)
 {
-	char *tocode = other->tocode;
-	tvterm_codingSystem_final(p);
-	if (strcmp(other->tocode, "UTF-8") == 0) {
+	static TCodingSystem otherCS;
+	char *tocode = ocs->tocode;
+	int i;
+
+	tvterm_finish_otherCS(p);
+	/* save current designate/invoke */
+	ocs->gSavedL = p->gl.invokedGn;
+	ocs->gSavedR = p->gr.invokedGn;
+	for (i = 0; i < 4; i++)
+		ocs->gSavedIdx[i] = p->gIdx[i];
+	ocs->utf8SavedIdx = p->utf8Idx;
+
+retry:
+	if (strcmp(ocs->tocode, "UTF-8") == 0) {
 		tvterm_switch_to_UTF8(p);
 		tocode = "UCS-2BE"; /* XXX */
 	} else {
 		int idx[6];
 		char *en;
-		int i;
 
-		en = tcaps_find_entry(p->caps, "encoding.", other->tocode);
-		if (en == NULL)
-			return;
-		if (tvterm_parse_encoding(en, idx) == 0)
-			return;
+		en = tcaps_find_entry(p->caps, "encoding.", ocs->tocode);
+		if (en == NULL) {
+			free(ocs->tocode);
+			ocs->tocode = strdup("UTF-8");
+			goto retry;
+		}
+
+		if (tvterm_parse_encoding(en, idx) == 0) {
+			free(ocs->tocode);
+			ocs->tocode = strdup("UTF-8");
+			goto retry;
+		}
 		for (i = 0; i < 4; i++)
 			p->gIdx[i] = idx[2+i];
 		tvterm_invoke_gx(p, &(p->gl), idx[0]);
@@ -304,53 +337,36 @@ tvterm_switch_to_other(TVterm *p, TCodingSystem *other)
 		p->tgr = p->gr;
 		p->knj1 = 0;
 	}
-	other->cd = iconv_open(tocode, other->fromcode);
-	if (other->cd == (iconv_t)(-1)) {
-		tvterm_codingSystem_final(p);
-		return;
+	ocs->cd = iconv_open(tocode, ocs->fromcode);
+	if (ocs->cd == (iconv_t)(-1)) {
+		if (strcmp(ocs->tocode, "UTF-8") == 0) {
+			free(ocs->tocode);
+			ocs->tocode = strdup("UTF-8");
+			goto retry;
+		}
+		goto FINALIZE;
 	}
-	memset(other->inbuf, 0, sizeof(other->inbuf));
-	other->inbuflen = 0;
-	memset(other->outbuf, 0, sizeof(other->outbuf));
-	p->codingSystem = other;
+	/* ok */
+	memcpy(&otherCS, ocs, sizeof(otherCS));
+	memset(otherCS.inbuf, 0, sizeof(otherCS.inbuf));
+	otherCS.inbuflen = 0;
+	memset(otherCS.outbuf, 0, sizeof(otherCS.outbuf));
+	p->otherCS = &otherCS;
+	return;
+FINALIZE:
+	free(ocs->fromcode);
+	free(ocs->tocode);
+	ocs->fromcode = NULL;
+	ocs->tocode = NULL;
+	return;
 }
 
 static int
-tvterm_is_otherCodingSystem(TVterm *p)
+tvterm_is_otherCS(TVterm *p)
 {
-	return (p->codingSystem != NULL);
+	return (p->otherCS != NULL);
 }
 
-void
-tvterm_setup_otherCodingSystem(TCaps *caps, TCapValue* vp)
-{
-	TCsv farg;
-	const char *fc;
-	const char *ocs;
-	TCodingSystem *other;
-	char *en;
-	int i;
-	for (i = 0; vp; vp = vp->next) {
-		if (i >= MAX_OTHER_CODING_SYSTEM)
-			return;
-		tcsv_init(&farg, vp->value);
-		if (farg.cap != 2) {
-			print_message("other-coding-system: skipped\n");
-			goto FINALIZE;
-		}
-		fc = tcsv_get_token(&farg);
-		ocs = tcsv_get_token(&farg);
-		other = &gOtherCodingSystem[i];
-		other->fch = fc[0];
-		en = tcaps_find_entry(caps, "encoding.", ocs);
-		if (en == NULL)
-			goto FINALIZE;
-		if (tvterm_parse_other(en, other))
-			i++;
-	FINALIZE:
-		tcsv_final(&farg);
-	}
-}
 #endif
 
 
@@ -369,10 +385,10 @@ void tvterm_set_default_encoding(TVterm* p, const char* en)
 #ifdef JFB_OTHER_CODING_SYSTEM
 	if (strcmp(g, "other") == 0) {
 		/* other,<coding-system>,iconv,<std-coding-system> */
-		static TCodingSystem other;
-		if (tvterm_parse_other(en, &other) == 0)
+		static TCodingSystem otherCS;
+		if (tvterm_parse_otherCS(en, &otherCS) == 0)
 			goto FINALIZE;
-		tvterm_switch_to_other(p, &other);
+		tvterm_switch_to_otherCS(p, &otherCS);
 		goto FINALIZE;
 	}
 #endif
@@ -662,7 +678,7 @@ int tvterm_iso_C0_set(TVterm* p, u_char ch)
 	case ISO_VT:
 	case ISO_FF:
 #ifdef JFB_OTHER_CODING_SYSTEM
-		if (!tvterm_is_otherCodingSystem(p))
+		if (!tvterm_is_otherCS(p))
 #endif
 			tvterm_set_default_invoke_and_designate(p);
 		/* fail into next case */
@@ -768,7 +784,7 @@ tvterm_put_utf8_char(TVterm *p, u_char ch)
 
 #ifdef JFB_OTHER_CODING_SYSTEM
 int 
-tvterm_put_other_char(TVterm *p, u_char ch)
+tvterm_put_otherCS_char(TVterm *p, u_char ch)
 {
 	int rev;
 	char *inbuf;
@@ -778,58 +794,58 @@ tvterm_put_other_char(TVterm *p, u_char ch)
 	size_t s;
 	int i;
 
-	if (p->codingSystem->inbuflen >= sizeof(p->codingSystem->inbuf)-1) {
+	if (p->otherCS->inbuflen >= sizeof(p->otherCS->inbuf)-1) {
 		/* XXX: too long? */
-		p->codingSystem->inbuflen = 0;
+		p->otherCS->inbuflen = 0;
 	}
-	p->codingSystem->inbuf[p->codingSystem->inbuflen] = ch;
-	p->codingSystem->inbuflen++;
+	p->otherCS->inbuf[p->otherCS->inbuflen] = ch;
+	p->otherCS->inbuflen++;
 
-	inbuf = p->codingSystem->inbuf;
-	inbuflen = p->codingSystem->inbuflen;
-	outbuf = p->codingSystem->outbuf;
-	outbuflen = sizeof(p->codingSystem->outbuf);
+	inbuf = p->otherCS->inbuf;
+	inbuflen = p->otherCS->inbuflen;
+	outbuf = p->otherCS->outbuf;
+	outbuflen = sizeof(p->otherCS->outbuf);
 
-	s = iconv(p->codingSystem->cd, &inbuf, &inbuflen, &outbuf, &outbuflen);
+	s = iconv(p->otherCS->cd, &inbuf, &inbuflen, &outbuf, &outbuflen);
 	if (s == (size_t)(-1)) {
 		switch (errno) {
 		case E2BIG:
-			p->codingSystem->inbuflen = 0;
+			p->otherCS->inbuflen = 0;
 			return 0;
 		case EILSEQ: /* illegal sequence */
-			p->codingSystem->inbuflen = 0;
+			p->otherCS->inbuflen = 0;
 			return 0;
 		case EINVAL: /* incomplete multibyte */
 			return 0;
 		}
 	}
 	/* output bytes */
-	s = sizeof(p->codingSystem->outbuf) - outbuflen;
+	s = sizeof(p->otherCS->outbuf) - outbuflen;
 
-	if (strcmp(p->codingSystem->tocode, "UTF-8") == 0) {
+	if (strcmp(p->otherCS->tocode, "UTF-8") == 0) {
 		p->ucs2ch = 0;
 		for (i = 0; i < s; i++) {
 			p->ucs2ch <<= 8;
-			p->ucs2ch |= (p->codingSystem->outbuf[i] & 0x0ff);
+			p->ucs2ch |= (p->otherCS->outbuf[i] & 0x0ff);
 		}
 
 		rev = tvterm_put_uchar(p, p->ucs2ch);
 		if (rev < 0) {
-			p->codingSystem->inbuflen--;
+			p->otherCS->inbuflen--;
 			return rev;
 		}
-		p->codingSystem->inbuflen = 0;
+		p->otherCS->inbuflen = 0;
 		return rev;
 	} else {
 		for (i = 0; i < s; i++) {
-			u_char c = p->codingSystem->outbuf[i];
+			u_char c = p->otherCS->outbuf[i];
 			rev = tvterm_put_normal_char(p, c);
 			if (rev < 0) {
-				p->codingSystem->inbuflen--;
+				p->otherCS->inbuflen--;
 				return rev;
 			}
 		}
-		p->codingSystem->inbuflen = 0;
+		p->otherCS->inbuflen = 0;
 		return 0;
 	}
 }
@@ -853,8 +869,8 @@ void tvterm_emulate(TVterm* p, const char *buff, int nchars)
 				continue;
 			}
 #ifdef JFB_OTHER_CODING_SYSTEM
-		} else if (tvterm_is_otherCodingSystem(p)) {
-			rev = tvterm_put_other_char(p, ch);
+		} else if (tvterm_is_otherCS(p)) {
+			rev = tvterm_put_otherCS_char(p, ch);
 			if (rev >= 0) {
 				continue;
 			} else if (rev < 0) {
@@ -909,109 +925,114 @@ static void tvterm_esc_start(TVterm* p, u_char ch)
 {
 	p->esc = NULL;
 	switch(ch) {
-	case Fe(ISO_CSI):
+	case Fe(ISO_CSI):	/* 5/11 [ */
 		p->esc = tvterm_esc_bracket;
 		break;
-	case ISO__MBS:
+#ifdef JFB_OTHER_CODING_SYSTEM
+	case Fe(ISO_OSC):	/* 5/13 ] */
+		p->esc = tvterm_esc_rbracket;
+		break;
+#endif
+	case ISO__MBS:		/* 2/4 $ */
 		p->esc = tvterm_esc_traditional_multibyte_fix;
 		p->escSignature = TFONT_FT_DOUBLE;
 		break;
-	case ISO_DOCS:
-		p->esc = tvterm_esc_designate_other_coding_system;
+	case ISO_DOCS:		/* 2/5 % */
+		p->esc = tvterm_esc_designate_otherCS;
 		break;
-	case ISO_GZD4:
+	case ISO_GZD4:		/* 2/8 ( */
 		ESC_ISO_GnDx(0, TFONT_FT_94CHAR);
 		break;
-	case MULE__GZD6:
+	case MULE__GZD6:	/* 2/12 , */
 		ESC_ISO_GnDx(0, TFONT_FT_96CHAR);
 		break;
-	case ISO_G1D4:
+	case ISO_G1D4:		/* 2/9 ) */
 		ESC_ISO_GnDx(1, TFONT_FT_94CHAR);
 		break;
-	case ISO_G1D6:
+	case ISO_G1D6:		/* 2/13 - */
 		ESC_ISO_GnDx(1, TFONT_FT_96CHAR);
 		break;
-	case ISO_G2D4:
+	case ISO_G2D4:		/* 2/10 * */
 		ESC_ISO_GnDx(2, TFONT_FT_94CHAR);
 		break;
-	case ISO_G2D6:
+	case ISO_G2D6:		/* 2/14 . */
 		ESC_ISO_GnDx(2, TFONT_FT_96CHAR);
 		break;
-	case ISO_G3D4:
+	case ISO_G3D4:		/* 2/11 + */
 		ESC_ISO_GnDx(3, TFONT_FT_94CHAR);
 		break;
-	case ISO_G3D6:
+	case ISO_G3D6:		/* 2/15 / */
 		ESC_ISO_GnDx(3, TFONT_FT_96CHAR);
 		break;
-	case Fe(ISO_NEL):		/* E */
+	case Fe(ISO_NEL):	/* 4/5 E */
 		p->pen.x = 0;
 		p->wrap = FALSE;
-	case Fe(TERM_IND):		/* D */
+	case Fe(TERM_IND): 	/* 4/4 D */
 		if (p->pen.y == p->ymax -1) {
 			p->scroll ++;
 		} else {
 			p->pen.y ++;
 		}
 		break;
-	case Fe(ISO_RI):		/* M */
+	case Fe(ISO_RI):	/* 4/13 M */
 		if (p->pen.y == p->ymin) {
 			p->scroll --;
 		} else {
 			p->pen.y --;
 		}
 		break;
-	case Fe(ISO_SS2):		/* N *//* 7bit single shift 2 */
+	case Fe(ISO_SS2):	/* 4/14 N *//* 7bit single shift 2 */
 		tvterm_invoke_gx(p, &(p->tgl), 2); /* GL <== G2 */
 		break;
-	case Fe(ISO_SS3):		/* O *//* 7bit single shift 3 */
+	case Fe(ISO_SS3):	/* 4/15 O *//* 7bit single shift 3 */
 		tvterm_invoke_gx(p, &(p->tgl), 3); /* GL <== G3 */
 		break;
 
-	case ISO_RIS:			/* c */
+	case ISO_RIS:		/* 6/3 c */
 		p->pen.fcol = 7;
 		p->pen.bcol = 0;
 		p->pen.attr = 0;
 		p->wrap = FALSE;
 		tvterm_set_default_invoke_and_designate(p);
 		/* fail into next case */
-/*	case TERM_CAH: - conflicts with ISO_G2D4, no terminfo */
+		/* TERM_CAH: - conflicts with ISO_G2D4, no terminfo */
 		p->pen.x = 0;
 		p->pen.y = 0;
 		p->wrap = FALSE;
 		tvterm_text_clear_all(p);
 		break;
-	case DEC_SC:
+	case DEC_SC:		/* 3/7 7 */
 		tvterm_push_current_pen(p, TRUE);
 		break;
-	case DEC_RC:
+	case DEC_RC:		/* 3/8 8 */
 		tvterm_pop_pen_and_set_currnt_pen(p, TRUE);
 		p->wrap = FALSE;
 		break;
-	case ISO_LS2:
+	case ISO_LS2:		/* 6/14 n */
 		if (!tvterm_is_ISO2022(p))
 			break;
 		tvterm_invoke_gx(p, &(p->gl), 2); /* GL <= G2 */
 		p->tgl = p->gl;
 		break;
-	case ISO_LS3:
+	case ISO_LS3:		/* 6/15 o */
 		if (!tvterm_is_ISO2022(p))
 			break;
 		tvterm_invoke_gx(p, &(p->gl), 3); /* GL <= G3 */
 		p->tgl = p->gl;
 		break;
-	case ISO_LS3R:
+	case ISO_LS3R:		/* 7/12 | */
 		if (!tvterm_is_ISO2022(p))
 			break;
 		tvterm_invoke_gx(p, &(p->gr), 3); /* GR <= G3 */
 		p->tgr = p->gr;
 		break;
-	case ISO_LS2R:
+	case ISO_LS2R:		/* 7/13 } */
 		if (!tvterm_is_ISO2022(p))
 			break;
 		tvterm_invoke_gx(p, &(p->gr), 2); /* GR <= G2 */
 		p->tgr = p->gr;
 		break;
-	case ISO_LS1R:
+	case ISO_LS1R:		/* 7/14 ~ */
 		if (!tvterm_is_ISO2022(p))
 			break;
 		tvterm_invoke_gx(p, &(p->gr), 1); /* GR <= G1 */
@@ -1023,6 +1044,8 @@ static void tvterm_esc_start(TVterm* p, u_char ch)
 /*---------------------------------------------------------------------------*/
 void tvterm_esc_set_attr(TVterm* p, int col)
 {
+	static int acsIdx = 0;
+
 	switch(col) {
 	case 0: tpen_off_all_attribute(&(p->pen));
 		break;
@@ -1038,27 +1061,23 @@ void tvterm_esc_set_attr(TVterm* p, int col)
 		break;
 	case 27: tpen_no_reverse(&(p->pen));
 		break;
-#if 0 /* FIX ME ! */
-	case 10: p->trans = CS_LEFT;
-		break;
-	case 11: p->trans = CS_GRAPH;
-		break;
-#else
-	case 10:
+	case 10:	/* rmacs, rmpch */
 		p->gIdx[0] = 0;
 		tvterm_re_invoke_gx(p, &(p->gl));
 		tvterm_re_invoke_gx(p, &(p->gr));
 		p->tgl = p->gl; p->tgr = p->gr;
 		break;
-#if 0
-	case 11:
-		p->gIdx[1] = 3;
-		tvterm_re_invoke_gx(p, &(p->gl));
-		tvterm_re_invoke_gx(p, &(p->gr));
-		p->tgl = p->gl; p->tgr = p->gr;
+	case 11:	/* smacs, smpch */
+		if (acsIdx == 0) {
+			acsIdx = tvterm_find_font_index(0x30|TFONT_FT_94CHAR);
+		}
+		if (acsIdx > 0) {
+			p->gIdx[0] = acsIdx;
+			tvterm_re_invoke_gx(p, &(p->gl));
+			tvterm_re_invoke_gx(p, &(p->gr));
+			p->tgl = p->gl; p->tgr = p->gr;
+		}
 		break;
-#endif
-#endif
 	default: tpen_set_color(&(p->pen), col);
 		break;
 	}
@@ -1306,6 +1325,54 @@ static void tvterm_esc_bracket(TVterm* p, u_char ch)
 	}
 }
 
+#ifdef JFB_OTHER_CODING_SYSTEM
+
+#define MAX_ARGLEN	48
+
+/*
+ * ESC  ]    p... F
+ * 1/11 5/13 p... F
+ *
+ * p: 2/0 - 7/E
+ * F: 0/1 - 1/15
+ *
+ * 0/5	designate private coding system
+ *
+ */
+static void
+tvterm_esc_rbracket(TVterm* p, u_char ch)
+{
+	static u_char arg[MAX_ARGLEN+1], enbuf[MAX_ARGLEN+32];
+	static int argidx;
+
+	if (ch >= 0x20 && ch <= 0x7e) {
+		if (argidx < MAX_ARGLEN-1)
+			arg[argidx++] = ch;
+		arg[argidx] = '\0';
+	} else {
+		const char *en;
+		TCodingSystem otherCS;
+
+		p->esc = NULL;
+		switch (ch) {
+		case 0x05:	/* 0/5 designate private coding system */
+			en = tcaps_find_entry(p->caps, "encoding.", arg);
+			if (en == NULL) {
+				sprintf(enbuf, "other,%s,iconv,UTF-8",
+					arg);
+				en = enbuf;
+			}
+			if (tvterm_parse_otherCS(en, &otherCS)) {
+				tvterm_switch_to_otherCS(p, &otherCS);
+			}
+			break;
+		}
+		argidx = 0;
+		memset(arg, 0, sizeof(arg));
+	}
+}
+#endif
+
 /* Note:
 * KON2 Support
 * ESC ISO_94_TO_G0 U  ==> GRAPHIC FONT SET
@@ -1328,6 +1395,17 @@ static void tvterm_re_invoke_gx(TVterm* p, TFontSpec* fs)
 	fs->half = gFont[fs->idx].fhalf;
 }
 
+static int tvterm_find_font_index(int fsig)
+{
+	int i;
+	for (i = 0; gFont[i].fsignature; i++) {
+		if (gFont[i].fsignature == fsig) {
+			return i;
+		}
+	}
+	return -1;
+}
+
 static void tvterm_esc_designate_font(TVterm* p, u_char ch)
 {
 	int i;
@@ -1336,19 +1414,16 @@ static void tvterm_esc_designate_font(TVterm* p, u_char ch)
 		return;
 	}
 
-	for (i = 0 ; gFont[i].fsignature ; i++) {
-		if (gFont[i].fsignature  == (ch | p->escSignature)) {
-			p->gIdx[p->escGn] = i;
-			tvterm_re_invoke_gx(p, &(p->gl));
-			tvterm_re_invoke_gx(p, &(p->gr));
-			p->tgl = p->gl; p->tgr = p->gr;
-			break;
-		}
+	if ((i = tvterm_find_font_index(ch | p->escSignature)) >= 0) {
+		p->gIdx[p->escGn] = i;
+		tvterm_re_invoke_gx(p, &(p->gl));
+		tvterm_re_invoke_gx(p, &(p->gr));
+		p->tgl = p->gl; p->tgr = p->gr;
 	}
 	p->esc = NULL;
 }
 
-static void tvterm_esc_designate_other_coding_system(TVterm* p, u_char ch)
+static void tvterm_esc_designate_otherCS(TVterm* p, u_char ch)
 {
 	switch (ch) {
 	case 0x40:
@@ -1359,20 +1434,6 @@ static void tvterm_esc_designate_other_coding_system(TVterm* p, u_char ch)
 	case 0x47: /* XXX: designate and invoke UTF-8 coding */
 		tvterm_switch_to_UTF8(p);
 		break;
-#endif
-#ifdef JFB_OTHER_CODING_SYSTEM
-	default:
-	{
-		int i;
-		/* XXX: other coding system? */
-		for (i = 0; i < MAX_OTHER_CODING_SYSTEM; i++) {
-			if (ch == gOtherCodingSystem[i].fch) {
-				tvterm_switch_to_other(p, 
-						       &gOtherCodingSystem[i]);
-				break;
-			}
-		}
-	}
 #endif
 	}
 	p->esc = NULL;
@@ -1388,7 +1449,6 @@ static void tvterm_esc_traditional_multibyte_fix(TVterm* p, u_char ch)
 }
 
 
-
 void tvterm_show_sequence(FILE *tf, TCaps *cap, const char *en)
 {
 	static const char *invokeL[] = {"\017", "\016", "\033n", "\033o"};
@@ -1404,32 +1464,11 @@ void tvterm_show_sequence(FILE *tf, TCaps *cap, const char *en)
 	g = tcsv_get_token(&farg);
 #ifdef JFB_OTHER_CODING_SYSTEM
 	if (strcmp(g, "other") == 0) {
-		TCodingSystem tcs;
-		memset(&tcs, 0, sizeof(tcs));
-		if (tvterm_parse_other(en, &tcs)) {
-			TCapability* fother;
-			TCapValue* vp;
-			fother = tcaps_find(cap, "other-coding-system");
-			if (!fother || !(fother->values))
-				goto end;
-			for (vp = fother->values; vp ; vp = vp->next) {
-				TCsv fo;
-				const char *fc;
-				const char *ocs;
-				tcsv_init(&fo, vp->value);
-				if (fo.cap != 2) {
-					tcsv_final(&fo);
-					continue;
-				}
-				fc = tcsv_get_token(&fo);
-				ocs = tcsv_get_token(&fo);
-				if (strcmp(tcs.fromcode, ocs) == 0) {
-					fprintf(tf, "%s%c", "\033%", fc[0]);
-					tcsv_final(&fo);
-					break;
-				}
-				tcsv_final(&fo);
-			}
+		TCodingSystem otherCS;
+		if (tvterm_parse_otherCS(en, &otherCS)) {
+			fprintf(tf, "%s%s\005", "\033]", otherCS.fromcode);
+			free(otherCS.fromcode);
+			free(otherCS.tocode);
 		}
 		goto end;
 	}
