@@ -141,6 +141,23 @@ static inline void bzero2(void *head, int n)
 #endif
 }
 
+static inline void memset2(void *head, u_char ch, int n)
+{
+#ifdef __i386__
+	int dummy1;
+	int dummy2;
+	__asm__ __volatile__ (
+		"cld\n\t"
+		"rep\n\t"
+		"stosb"
+		:"=c" (dummy1), "=D" (dummy2)
+		:"a" (ch), "0" (n), "1" ((long)head)
+	);
+#else
+    memset(head,ch,n);
+#endif
+}
+
 static inline void lmove(void *dst, void *src, int n)
 {
 #ifdef __i386__
@@ -176,6 +193,24 @@ static inline void lzero(void *head, int n)
 #endif
 }
 
+static inline void lset(void *head, u_char ch, int n)
+{
+#ifdef __i386__
+	int dummy1;
+	int dummy2;
+	__asm__ __volatile__ (
+		"cld\n\t"
+		"rep\n\t"
+		"stosl"
+		:"=c" (dummy1), "=D" (dummy2)
+		:"a" (ch<<24|ch<<16|ch<<8|ch), "0" (n>>2), "1" ((long)head)
+	);
+#else
+    memset(head,ch,n);
+#endif
+}
+
+
 static inline void blatch(void *head, int n)
 {
 #ifdef __i386__
@@ -241,18 +276,18 @@ static inline void tvterm_brmove(TVterm* p, int dst, int src, int n)
 	brmove(p->flag+dst, p->flag+src, n);
 }
 
-static inline void tvterm_bzero2(TVterm* p, int top, int n)
+static inline void tvterm_clear(TVterm* p, int top, int n)
 {
 	bzero2(p->text+top, n*sizeof(u_int));
-	bzero2(p->attr+top, n);
 	bzero2(p->flag+top, n);
+	memset2(p->attr+top, p->pen.bcol, n);
 }
 
-static inline void tvterm_lzero(TVterm* p, int top, int n)
+static inline void tvterm_lclear(TVterm* p, int top, int n)
 {
 	lzero(p->text+top, n*sizeof(u_int));
-	lzero(p->attr+top, n);
 	lzero(p->flag+top, n);
+	lset(p->attr+top, p->pen.bcol, n);
 }
 /*---------------------------------------------------------------------------*/
 void tvterm_delete_n_chars(TVterm* p, int n)
@@ -267,7 +302,7 @@ void tvterm_delete_n_chars(TVterm* p, int n)
 	blatch(p->flag+addr, dx);
 	
 	addr = tvterm_coord_to_index(p, p->xcap-n, p->pen.y);
-	tvterm_bzero2(p, addr, n);
+	tvterm_clear(p, addr, n);
 }
 
 void tvterm_insert_n_chars(TVterm* p, int n)
@@ -281,7 +316,7 @@ void tvterm_insert_n_chars(TVterm* p, int n)
 	
 	addr = tvterm_coord_to_index(p, p->pen.x, p->pen.y);
 	blatch(p->flag+addr+n, dx);
-	tvterm_bzero2(p, addr, n);
+	tvterm_clear(p, addr, n);
 }
 
 #if 0 /* ϡɥ뤹ݤ˻Ȥ(̤) */
@@ -296,12 +331,12 @@ static void tvterm_scroll_up_n_lines(TVterm* p, int n)
 		p->textHead -= p->tsize;
 		n = p->tsize - h;
 		if (p->textHead) {
-			tvterm_lzero(p, 0, p->textHead);
+			tvterm_lclear(p, 0, p->textHead);
 		}
 	} else  {
 		n = p->textHead - h;
 	}
-	tvterm_lzero(p, h, n);
+	tvterm_lclear(p, h, n);
 }
 
 static void tvterm_scroll_down_n_lines(TVterm* p, int n)
@@ -314,13 +349,13 @@ static void tvterm_scroll_down_n_lines(TVterm* p, int n)
 	if (p->textHead < 0) {
 		p->textHead += p->tsize;
 		if (h) {
-			tvterm_lzero(p, 0, h);
+			tvterm_lclear(p, 0, h);
 		}
 		n = p->tsize - p->textHead;
 	} else {
 		n = h - p->textHead;
 	}
-	tvterm_lzero(p, h, n);
+	tvterm_lclear(p, h, n);
 }
 #endif
 
@@ -573,7 +608,7 @@ void tvterm_text_clear_all(TVterm* p)
 
 	for (y = 0; y < p->ymax; y++) {
 		a = tvterm_coord_to_index(p, 0, y);
-		tvterm_lzero(p, a, p->xcap4);
+		tvterm_lclear(p, a, p->xcap4);
 	}
 	p->textClear = TRUE;
 }
@@ -604,7 +639,7 @@ void tvterm_text_clear_eol(TVterm* p, u_char mode)
 		break;
 	}
 	a = tvterm_coord_to_index(p, x, p->pen.y);
-	tvterm_bzero2(p, a, len);
+	tvterm_clear(p, a, len);
 }
 
 /**
@@ -625,7 +660,7 @@ void tvterm_text_clear_eos(TVterm* p, u_char mode)
 	case 1:
 		tvterm_text_clean_band(p, 0, p->pen.y);
 		a = tvterm_coord_to_index(p, 0, p->pen.y);
-		tvterm_bzero2(p, a, p->pen.x);
+		tvterm_clear(p, a, p->pen.x);
 		break;
 	case 2:
 		tvterm_text_clear_all(p);
@@ -634,7 +669,7 @@ void tvterm_text_clear_eos(TVterm* p, u_char mode)
 		tvterm_text_clean_band(p, p->pen.y + 1, p->ycap);
 		a = tvterm_coord_to_index(p, p->pen.x, p->pen.y);
 		len = p->xcap - p->pen.x;
-		tvterm_bzero2(p, a, len);
+		tvterm_clear(p, a, len);
 		break;
 	}
 }
@@ -649,7 +684,7 @@ static void tvterm_text_clean_band(TVterm* p, u_int top, u_int btm)
 	
 	for (y = top; y < btm; y ++) {
 		a = tvterm_coord_to_index(p, 0, y);
-		tvterm_lzero(p, a, p->xcap4);
+		tvterm_lclear(p, a, p->xcap4);
 		/* needless to latch */
 	}
 }
